package de.ullisroboterseite.ursai2actionbar;

// Autor: https://UllisRoboterSeite.de

// Doku:  https://UllisRoboterSeite.de/android-AI2-ActionBar.html
//
// Version 1.0 (2021-01-17)
// -------------------------
// - Basis-Version
//
// Version 1.1 (2021-01-18)
// -------------------------
// - Zuordnung der Buttons (#1 & #2) getauscht.
// - LeftButtonColor hat nicht funktioniert
//
// Version 2.0 (2021-01-22)
// -------------------------
// - Popup-Menü ist eigensändig
//
//
// Version 2.1 (2021-01-28)
// -------------------------
// - Die Symbolnamen können direkt aus https://material.io/resources/icons angegeben werden
// - Die Umrechnung in Unicode-Symbole entfällt.
//
// Version 2.1 (2022-02-20)
// -------------------------
// - Symbolnamen (z.B. LeftButtonSymbol) können zusätzlich den Text "blank" annehmen. Dann wird ein Leerzeichen angezeigt.
// - Über Eigenschaften LeftButtonImage und RightButtonImage kann ein Asset mit einer Bild-Datei als Hintergrund eingestellt werden.



import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.util.Log;
import android.os.Handler;
import android.graphics.Typeface;
import android.content.Context;

@DesignerComponent(version = 1, //
        versionName = UrsAI2ActionBar.VersionName, //
        dateBuilt = "2022-02-20", //
        description = "AI2 extension ActionBar component.", //
        category = com.google.appinventor.components.common.ComponentCategory.EXTENSION, //
        nonVisible = true, //
        helpUrl = "https://UllisRoboterSeite.de/android-AI2-ActionBar.html", //
        iconName = "aiwebres/icon.png")
@UsesAssets(fileNames = "MaterialIcons-Regular.ttf")
@SimpleObject(external = true)
public class UrsAI2ActionBar extends AndroidNonvisibleComponent implements OnInitializeListener {
    static final String LOG_TAG = "MENU";
    static final String VersionName = "2.2.1";

    final Context context;
    Handler handler = new Handler();

    final ActionBar actionBar;
    final Popup popup;

    HorizontalArrangement placeholder = null;



    public UrsAI2ActionBar(ComponentContainer container) {
        super(container.$form());
        context = container.$context();

        form.registerForOnInitialize(this);

        Typeface font = null;

        String fontPath = "";
        try {
            fontPath = form.getAssetPathForExtension(this, "MaterialIcons-Regular.ttf");
            if (fontPath.startsWith(Form.ASSETS_PREFIX)) {
                font = Typeface.createFromAsset(form.getAssets(), fontPath.substring(Form.ASSETS_PREFIX.length()));
            } else {
                font = Typeface.createFromFile(fontPath.substring(("file://").length()));
            }
        } catch (Exception e) {
            // nichts tun
        }

        actionBar = new ActionBar(this, form, font);
        popup = new Popup(this, form);
    }

    /**
    * Wird vom übergeordneten Form aufgerufen nachdem das Ereignis Screen.Initialize ausgelöst wurde.
    */
    @Override
    public void onInitialize() {
        actionBar.InitActionBar(placeholder);
    }

    @SimpleProperty(description = "Returns the component's version name.")
    public String Version() {
        return VersionName;
    }

    // ====================================================
    // Allgemein /Hintergrund
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COMPONENT, defaultValue = "")
    @SimpleProperty(description = "Placeholder component (HorizontalArrangement)", userVisible = false)
    public void Placeholder(AndroidViewComponent viewComponent) {
        if (viewComponent instanceof HorizontalArrangement) {
            placeholder = (HorizontalArrangement) viewComponent;
        } else {
            form.ErrorOccurred(this, "PlaceHolder", 7777, "Component not of Type HorizontalArrangement");
        }
    }

    // ====================================================
    // Titel
    // ====================================================

    @SimpleProperty(description = "Sets the title of the action bar.")
    @DesignerProperty(defaultValue = "Screen", editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING)
    public void Title(String value) {
        actionBar.setTitle(value);
    }

    @SimpleProperty(description = "Gets the title of the action bar.")
    public String Title() {
        return actionBar.getTitle();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_NON_NEGATIVE_FLOAT, defaultValue = "18.0")
    @SimpleProperty(description = "Specifies the title's font size, measured in sp(scale-independent pixels).")
    public void FontSize(float size) {
        actionBar.setFontSize(size);
    }

    @SimpleProperty(description = "Gets the title of the action bar.")
    public float FontSize() {
        return actionBar.getFontSize();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "True")
    @SimpleProperty(description = "Specifies whether the titles's text should be bold.")
    public void FontBold(boolean value) {
        actionBar.setFontBold(value);
    }

    @SimpleProperty(description = "Returns true if the titles's text should be bold.")
    public boolean FontBold() {
        return actionBar.getFontBold();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the titles's text color as an alpha-red-green-blue integer.")
    public void TextColor(int argb) {
        actionBar.setTextColor(argb);
    }

    @SimpleProperty(description = "Returns the titles's text color as an alpha-red-green-blue integer")
    @IsColor
    public int TextColor() {
        return actionBar.getTextColor();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = "&HFF4862A3")
    @SimpleProperty(description = "Specifies the Actionbar's background color as an alpha-red-green-blue integer")
    public void BackgroundColor(int argb) {
        actionBar.setBackgroundColor(argb);
    }

    @SimpleProperty(description = "Returns the label's Actionbar's color as an alpha-red-green-blue integer.")
    @IsColor
    public int BackgroundColor() {
        return actionBar.getBackgroundColor();
    }

    @SimpleProperty(description = "Returns the alignment of the titles's text", userVisible = false)
    public int TextAlignment() {
        return actionBar.getTextAlignment();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXTALIGNMENT, defaultValue = Component.ALIGNMENT_NORMAL
            + "")
    @SimpleProperty(description = "Specifies the alignment of the titles's text", userVisible = false)
    public void TextAlignment(int alignment) {
        actionBar.setTextAlignment(alignment);
    }

    // ====================================================
    // Menü
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXTAREA, defaultValue = "")
    @SimpleProperty(description = "Set the list of choices from a string of comma-separated values.")
    public void MenuItemsFromString(String itemstring) {
        popup.setMenuItems(ElementsUtil.elementsFromString(itemstring));
    }

    @SimpleFunction(description = "Loads the menu items from a list.")
    public void SetMenuItems(final YailList ItemList) {
        popup.setMenuItems(ItemList);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
    @SimpleProperty(description = "Loads the menu items from a file.", userVisible = false)
    public void MenuItemsFromFile(String FileName) {
        popup.loadMenuItemsFromFile(FileName);
    }

    @SimpleFunction(description = "Loads the menu items from a file.", userVisible = false)
    public void LoadMenuItemsFromFile(String FileName) {
        popup.loadMenuItemsFromFile(FileName);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_NON_NEGATIVE_INTEGER, defaultValue = "16")
    @SimpleProperty(description = "Specifies the title's font size, measured in sp(scale-independent pixels).")
    public void MenuFontSize(int size) {
        popup.setMenuFontSize(size);
    }

    @SimpleProperty(description = "Gets the title of the action bar.")
    public int MenuFontSize() {
        return popup.getMenuFontSize();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_CHOICES, defaultValue = "Default", //
            editorArgs = { "Default", "White", "Black" })
    @SimpleProperty(description = "Sets the menu's backgroun color.")
    public void MenuBackgroundColor(String value) {
        switch (value.toLowerCase()) {
            case "white":
                popup.setMenuBackgroundColor(1);
                break;
            case "black":
            popup.setMenuBackgroundColor(2);
                break;
            default:
            popup.setMenuBackgroundColor(0);
                break;
        }
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the menu's text color as an alpha-red-green-blue integer.")
    public void MenuTextColor(int argb) {
        popup.setMenuTextColor(argb);
    }

    @SimpleProperty(description = "Returns the menu's text color as an alpha-red-green-blue integer.")
    @IsColor
    public int MenuTextColor() {
        return popup.getMenuTextColor();
    }

    // ====================================================
    // Left Button
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "menu")
    @SimpleProperty(description = "Sets the left button's symbol.")
    public void LeftButtonSymbol(String value) {
        actionBar.setLeftButtonSymbol(value);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the text color foe button #1 as an alpha-red-green-blue integer.")
    public void LeftButtonColor(int argb) {
        actionBar.setLeftButtonTextColor(argb);
    }

    @SimpleProperty(description = "Returns the text color of Left button as an alpha-red-green-blue integer.")
    @IsColor
    public int LeftButtonColor() {
        return actionBar.getLeftButtonTextColor();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
    @SimpleProperty(description = "A small icon displayed in the left button's background.")
    public void LeftButtonImage(String value) {
        actionBar.setLeftButtonImage(value.trim());
    }

    @SimpleProperty(description = "A small icon displayed in the left button's background.")
    public String LeftButtonImage() {
       return actionBar.getLeftButtonImage();
    }


    // ====================================================
    // Right Button
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "more_vert")
    @SimpleProperty(description = "Sets the right button's symbol.")
    public void RightButtonSymbol(String value) {
        actionBar.setRightButtonSymbol(value);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the text color foe button #1 as an alpha-red-green-blue integer.")
    public void RightButtonColor(int argb) {
        actionBar.setRightButtonTextColor(argb);
    }

    @SimpleProperty(description = "Returns the text color of right button as an alpha-red-green-blue integer.")
    @IsColor
    public int RightButtonColor() {
        return actionBar.getRightButtonTextColor();
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
    @SimpleProperty(description = "A small icon displayed in the right button's background.")
    public void RightButtonImage(String value) {
        actionBar.setRightButtonImage(value.trim());
    }

    @SimpleProperty(description = "A small icon displayed in the right button's background.")
    public String RightButtonImage() {
       return actionBar.getRightButtonImage();
    }

    // ====================================================
    // Button #1
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
    @SimpleProperty(description = "Sets the symbol of button 1.")
    public void Button1Symbol(String value) {
        actionBar.setButton1Symbol(value);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the text color foe button #1 as an alpha-red-green-blue integer.")
    public void Button1Color(int argb) {
        actionBar.setButton1TextColor(argb);
    }

    @SimpleProperty(description = "Returns the text color of button #1 as an alpha-red-green-blue integer")
    @IsColor
    public int Button1Color() {
        return actionBar.getButton1TextColor();
    }

    // ====================================================
    // Button #2
    // ====================================================

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
    @SimpleProperty(description = "Sets the symbol of button 2.")
    public void Button2Symbol(String value) {
        actionBar.setButton2Symbol(value);
    }

    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, defaultValue = Component.DEFAULT_VALUE_COLOR_WHITE)
    @SimpleProperty(description = "Specifies the text color foe button #2 as an alpha-red-green-blue integer.")
    public void Button2Color(int argb) {
        actionBar.setButton2TextColor(argb);
    }

    @SimpleProperty(description = "Returns the text color of button #2 as an alpha-red-green-blue integer")
    @IsColor
    public int Button2Color() {
        return actionBar.getButton2TextColor();
    }

    // =============================================================
    // Events
    // =============================================================

    @SimpleEvent(description = "Left button was clicked.")
    public void LeftButtonClick() {
        EventDispatcher.dispatchEvent(this, "LeftButtonClick");
    }

    @SimpleEvent(description = "Right button was clicked.")
    public void RightButtonClick() {
        EventDispatcher.dispatchEvent(this, "RightButtonClick");
        popup.show(actionBar.buttonRight);
    }

    @SimpleEvent(description = "Button #1 was clicked.")
    public void Button1Click() {
        EventDispatcher.dispatchEvent(this, "Button1Click");
    }

    @SimpleEvent(description = "Button #1 was clicked.")
    public void Button2Click() {
        EventDispatcher.dispatchEvent(this, "Button2Click");
    }

    @SimpleEvent(description = "Menu item was selected")
    public void MenuItemSelected(int ID, String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItemSelected", ID, Title, Checked);
    }

    @SimpleEvent(description = "Menu item #1 was selected")
    public void MenuItem1Selected(String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItem1Selected", Title, Checked);
    }

    @SimpleEvent(description = "Menu item #2 was selected")
    public void MenuItem2Selected(String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItem2Selected", Title, Checked);
    }

    @SimpleEvent(description = "Menu item #3 was selected")
    public void MenuItem3Selected(String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItem3Selected", Title, Checked);
    }

    @SimpleEvent(description = "Menu item #4 was selected")
    public void MenuItem4Selected(String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItem4Selected", Title, Checked);
    }

    @SimpleEvent(description = "Menu item #5 was selected")
    public void MenuItem5Selected(String Title, boolean Checked) {
        EventDispatcher.dispatchEvent(this, "MenuItem5Selected", Title, Checked);
    }
}
